#!/bin/sh
#
#---------------------------------------------------------------------------
# This bash shell script is used to process airborne (N42 & N43) radar Vr to SO.
# Original ascii airborne data is from: ftp://ftp.aoml.noaa.gov/pub/hrd/data/radar/ 
# Fuqing Zhang & Yonghui Weng, TAMU, Apr 2008
#---------------------------------------------------------------------------
# Notes:
#      : This script will make these dirs:
#      : ../Raw/. :  please saved the original data here
#      : ../YYMMDD/. : sweep files
#      : ../SO/.     : SO files 

  export obsday0=05
  export obsday1=08
  export obsday2=28
  export start_time0=17
  export start_time1=25
  export end_time0=18
  export end_time1=18
  export airborne="I1"
#---------------------------------------------------------------------------
  export workdir="~/SO/Raw"
  cd ${workdir}
#---------------------------------------------------------------------------
# 1. split the data into sweeps;
     export raw_flname="${obsday0}${obsday1}${obsday2}${airborne}_${start_time0}${start_time1}_${end_time0}${end_time1}"
     export raw_flname=$raw_flname"_radials"
     mkdir -p ${workdir}/../${obsday0}${obsday1}${obsday2}
#     ${workdir}/radials_split_sweep ${raw_flname}_radials
     rm ${workdir}/../${obsday0}${obsday1}${obsday2}/*
     export start_year=`expr ${obsday0} + 2000`
     export minutes=`expr ${end_time0} - ${start_time0}`
#     export minutes=`expr ${minutes} - 1`
     export minutes=`expr ${minutes} \* 60`
     export minutes=`expr ${minutes} + ${end_time1}`
     export minutes=`expr ${minutes} - ${start_time1}`
     echo "gunzipping radial txt file"
     gunzip $raw_flname".txt.gz"
     ${workdir}/../../bin/radials_split_sweeps noaa $raw_flname
     echo "gzipping radial txt file"
     gzip $raw_flname".txt"
#---------------------------------------------------------------------------
# 2. SO
     cd ${workdir}
     rm -f ${workdir}/namelist.so    
cat > ${workdir}/namelist.so << EOF
 &so_attribute
 elevmax = 70.,
 mindis  = 0.5,
 maxdis  = 75.,
 ddis    = 2.,
 dazm    = 3.,
 threshold_r = 2.0,
 threshold_a = 5.0,
 stdrv = 2.0,
 minrv = 0.,
 maxrv = 120.,
 /
 &so_time
 start_year  = ${start_year},
 start_month = ${obsday1},
 start_day   = ${obsday2},
 start_hour  = ${start_time0},
 start_minute =${start_time1},
 start_second = 00,
 minutes      = ${minutes},
 /
EOF
     ${workdir}/../../bin/rdsweep_to_so > print.out_${raw_flname}
     wait
     mkdir -p ../SO
     cat ${workdir}/../${obsday0}${obsday1}${obsday2}/${obsday0}${obsday1}${obsday2}*.so > ${workdir}/../SO/${raw_flname}.so
#---------------------------------------------------------------------------
# 3. random SO
     cd ${workdir}/../SO
     ${workdir}/../../bin/random_pick_so ../SO/${raw_flname}
     gzip *.so*
     mkdir -p save
     cp *.so*.gz save/
     cp *.so*.gz ../../ftpdir/
     rm *.so*.gz
     cd ${workdir}/../${obsday0}${obsday1}${obsday2}
     rm *.ass
     rm *.so
     rm *.swp
     echo "done!"
exit
